package bme.ar.labs.lab01;

import android.content.pm.ActivityInfo;
import android.support.v7.app.ActionBarActivity;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.SurfaceView;
import android.util.Log;
import org.opencv.android.OpenCVLoader;
import org.opencv.android.BaseLoaderCallback;
import org.opencv.android.LoaderCallbackInterface;
import org.opencv.core.CvType;
import org.opencv.core.Mat;
import org.opencv.android.CameraBridgeViewBase;
import org.opencv.android.CameraBridgeViewBase.CvCameraViewListener2;
import org.opencv.android.CameraBridgeViewBase.CvCameraViewFrame;
import org.opencv.imgcodecs.Imgcodecs;
import org.opencv.imgproc.Imgproc;

import java.io.File;
import android.os.Environment;
import org.opencv.core.Size;

public class MainActivity extends ActionBarActivity  implements CvCameraViewListener2{

    private BaseLoaderCallback mLoaderCallback = new BaseLoaderCallback(this) {
        @Override
        public void onManagerConnected(int status) {
            switch (status) {
                case LoaderCallbackInterface.SUCCESS: {
                    Log.i("OPenCVJavTEst", "OpenCV loaded successfully");
                    onOpenCVInited();
                }
                break;
                default: {
                    super.onManagerConnected(status);
                }
                break;
            }
        }
    };

    private void onOpenCVInited() {
        System.loadLibrary("NativeCodes");
        android.util.Log.e("", "native codes lib loaded");
        mOpenCvCameraView.enableView();
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        //setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
        //requestWindowFeature(Window.FEATURE_NO_TITLE);
        //getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN,
        //        WindowManager.LayoutParams.FLAG_FULLSCREEN);

        setContentView(R.layout.activity_main);
        mOpenCvCameraView = (CameraBridgeViewBase) findViewById(R.id.lab01_opencv_camera_view);
        mOpenCvCameraView.setVisibility(SurfaceView.VISIBLE);
        mOpenCvCameraView.setCvCameraViewListener(this);
    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onResume() {
        super.onResume();
        OpenCVLoader.initAsync(OpenCVLoader.OPENCV_VERSION_3_0_0, this,
                mLoaderCallback);
    }

    public void onPause()
    {
        super.onPause();
        if (mOpenCvCameraView != null)
            mOpenCvCameraView.disableView();
    }

    public void onDestroy() {
        super.onDestroy();
        if (mOpenCvCameraView != null)
            mOpenCvCameraView.disableView();
    }

    Mat mOutImage;
    Mat mTmpImage1;
    Mat mTmpImage2;
    public void onCameraViewStarted(int width, int height) {
        mOutImage = new Mat(height, width, CvType.CV_8UC4);
        mTmpImage1 = new Mat(height, width, CvType.CV_8UC4);
        mTmpImage2 = new Mat(height, width, CvType.CV_8UC4);

        mTmpImage1 = loadImageFromFile("test/markers.jpg");
        Imgproc.resize(mTmpImage1, mTmpImage2, new Size(width, height));

        mOutImage = loadImageFromFile("test/bme1.jpg");
        Imgproc.resize(mOutImage, mTmpImage1, new Size(width, height));
    }

    public Mat loadImageFromFile(String fileName) {

        Mat rgbLoadedImage = null;

        File root = Environment.getExternalStorageDirectory();
        File file = new File(root, fileName);

        // this should be in BGR format according to the
        // documentation.
        Mat image = Imgcodecs.imread(file.getAbsolutePath());

        if (image.width() > 0) {

            rgbLoadedImage = new Mat(image.size(), image.type());

            Imgproc.cvtColor(image, rgbLoadedImage, Imgproc.COLOR_BGR2RGBA);

            image.release();
            image = null;
        }
        else {
            Log.e("TEST", "failed to load image "+fileName+" ("+file.getAbsolutePath()+")");
        }

        return rgbLoadedImage;

    }

    public void onCameraViewStopped() {
    }

    public Mat onCameraFrame(CvCameraViewFrame inputFrame) {
        //new NativeHolder().edgeDetect(inputFrame.gray().getNativeObjAddr(), mTmpImage1.getNativeObjAddr());
        //Imgproc.cvtColor(mTmpImage1, mOutImage, Imgproc.COLOR_GRAY2RGBA, 4);
        //return mOutImage;
        //return inputFrame.rgba();

        //return mTmpImage2;

        new NativeHolder().placeImageToQuads(mTmpImage2.getNativeObjAddr(), mOutImage.getNativeObjAddr(), mTmpImage1.getNativeObjAddr());
        return mOutImage;
    }

    private CameraBridgeViewBase mOpenCvCameraView;
}
